#!/usr/bin/python

# Ubuntu Tweak - PyGTK based desktop configure tool
#
# Copyright (C) 2007-2008 TualatriX <tualatrix@gmail.com>
#
# Ubuntu Tweak is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Ubuntu Tweak is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ubuntu Tweak; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import os
import gtk
import gettext
import gobject
import pango

from common.consts import *
from common.utils import *
from common.appdata import APPS, CATES_DATA
from common.widgets import TweakPage
from common.widgets.dialogs import InfoDialog, ErrorDialog
from common.appdata import get_app_logo, get_app_describ
from xdg.DesktopEntry import DesktopEntry

try:
    from common.package import package_worker, PackageInfo
    DISABLE = False
except ImportError:
    DISABLE = True

DESKTOP_DIR = '/usr/share/app-install/desktop/'
ICON_DIR = os.path.join(DATA_DIR, 'applogos')

(
    COLUMN_INSTALLED,
    COLUMN_ICON,
    COLUMN_PKG,
    COLUMN_NAME,
    COLUMN_DESC,
    COLUMN_DISPLAY,
    COLUMN_CATE,
    COLUMN_TYPE,
) = range(8)

(
    CATE_ID,
    CATE_NAME,
    CATE_ICON,
) = range(3)

class AppView(gtk.TreeView):
    __gsignals__ = {
        'changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_INT,))
    }

    def __init__(self):
        gtk.TreeView.__init__(self)

        self.to_add = []
        self.to_rm = []
        self.filter = None

        model = self.__create_model()
        self.__add_columns()
        self.set_model(model)

        self.set_rules_hint(True)
        self.set_search_column(COLUMN_NAME)

        self.show_all()

    def __create_model(self):
        model = gtk.ListStore(
                        gobject.TYPE_BOOLEAN,
                        gtk.gdk.Pixbuf,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING)

        return model

    def sort_model(self):
        model = self.get_model()
        model.set_sort_column_id(COLUMN_NAME, gtk.SORT_ASCENDING)

    def __add_columns(self):
        renderer = gtk.CellRendererToggle()
        renderer.set_property("xpad", 6)
        renderer.connect('toggled', self.on_install_toggled)

        column = gtk.TreeViewColumn(' ', renderer, active = COLUMN_INSTALLED)
        column.set_cell_data_func(renderer, self.install_column_view_func)
        column.set_sort_column_id(COLUMN_INSTALLED)
        self.append_column(column)

        column = gtk.TreeViewColumn('Applications')
        column.set_sort_column_id(COLUMN_NAME)
        column.set_spacing(5)
        renderer = gtk.CellRendererPixbuf()
        column.pack_start(renderer, False)
        column.set_cell_data_func(renderer, self.icon_column_view_func)
        column.set_attributes(renderer, pixbuf = COLUMN_ICON)

        renderer = gtk.CellRendererText()
        renderer.set_property("xpad", 6)
        renderer.set_property("ypad", 6)
        renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
        column.pack_start(renderer, True)
        column.add_attribute(renderer, 'markup', COLUMN_DISPLAY)
        self.append_column(column)

    def install_column_view_func(self, cell_layout, renderer, model, iter):
        package = model.get_value(iter, COLUMN_PKG)
        if package == None:
            renderer.set_property("visible", False)
        else:
            renderer.set_property("visible", True)

    def icon_column_view_func(self, cell_layout, renderer, model, iter):
        icon = model.get_value(iter, COLUMN_ICON)
        if icon == None:
            renderer.set_property("visible", False)
        else:
            renderer.set_property("visible", True)

    def clear_model(self):
        self.get_model().clear()

    def append_app(self, status, pixbuf, pkgname, appname, desc, category, type='app'):
        model = self.get_model()

        model.append((status,
                pixbuf,
                pkgname,
                appname,
                desc,
                '<b>%s</b>\n%s' % (appname, desc),
                category,
                type))

    def append_changed_app(self, status, pixbuf, pkgname, appname, desc, category):
        model = self.get_model()

        model.append((status,
                pixbuf,
                pkgname,
                appname,
                desc,
                '<span foreground="#ffcc00"><b>%s</b>\n%s</span>' % (appname, desc),
                category,
                'app'))

    def append_update(self, status, pkgname, summary):
        model = self.get_model()
        self.to_add.append(pkgname)

        icontheme = gtk.icon_theme_get_default()
        for icon_name in ['application-x-deb', 'package-x-generic', 'package']:
            icon = icontheme.lookup_icon(icon_name, 32, gtk.ICON_LOOKUP_NO_SVG)
            if icon:
                break

        if icon:
            pixbuf = icon.load_icon()
        else:
            pixbuf = icontheme.load_icon(gtk.STOCK_MISSING_IMAGE, 32, 0)

        model.append((status,
                      pixbuf,
                      pkgname,
                      pkgname,
                      summary,
                      '<b>%s</b>\n%s' % (pkgname, summary),
                      None,
                      'update'))

    def update_model(self, apps, cates=None):
        '''apps is a list to iter pkgname,
        cates is a dict to find what the category the pkg is
        '''
        def do_append(is_installed, pixbuf, pkgname, appname, desc, category):
            if pkgname in self.to_add or pkgname in self.to_rm:
                self.append_changed_app(not is_installed,
                        pixbuf,
                        pkgname,
                        appname,
                        desc,
                        category)
            else:
                self.append_app(is_installed,
                        pixbuf,
                        pkgname,
                        appname,
                        desc,
                        category)

        model = self.get_model()

        icon = gtk.icon_theme_get_default()

        for pkgname in apps:
            if cates:
                category = cates[pkgname][0]
            else:
                category = 0

            pixbuf = get_app_logo(pkgname)

            try:
                package = PackageInfo(pkgname)
                is_installed = package.check_installed()
                appname = package.get_name()
                desc = get_app_describ(pkgname)
            except KeyError:
                continue

            if self.filter == None:
                do_append(is_installed, pixbuf, pkgname, appname, desc, category)
            else:
                if self.filter == category:
                    do_append(is_installed, pixbuf, pkgname, appname, desc, category)

    def on_install_toggled(self, cell, path):
        def do_app_changed(model, iter, appname, desc):
                model.set(iter, COLUMN_DISPLAY, '<span style="italic" weight="bold"><b>%s</b>\n%s</span>' % (appname, desc))
        def do_app_unchanged(model, iter, appname, desc):
                model.set(iter, COLUMN_DISPLAY, '<b>%s</b>\n%s' % (appname, desc))

        model = self.get_model()

        iter = model.get_iter((int(path),))
        is_installed = model.get_value(iter, COLUMN_INSTALLED)
        pkgname = model.get_value(iter, COLUMN_PKG)
        appname = model.get_value(iter, COLUMN_NAME)
        desc = model.get_value(iter, COLUMN_DESC)
        type = model.get_value(iter, COLUMN_TYPE)

        if type == 'app':
            is_installed = not is_installed
            if is_installed:
                if pkgname in self.to_rm:
                    self.to_rm.remove(pkgname)
                    do_app_unchanged(model, iter, appname, desc)
                else:
                    self.to_add.append(pkgname)
                    do_app_changed(model, iter, appname, desc)
            else:
                if pkgname in self.to_add:
                    self.to_add.remove(pkgname)
                    do_app_unchanged(model, iter, appname, desc)
                else:
                    self.to_rm.append(pkgname)
                    do_app_changed(model, iter, appname, desc)

            model.set(iter, COLUMN_INSTALLED, is_installed)
        else:
            to_installed = is_installed
            to_installed = not to_installed
            if to_installed == True:
                self.to_add.append(pkgname)
            else:
                self.to_add.remove(pkgname)

            model.set(iter, COLUMN_INSTALLED, to_installed)

        self.emit('changed', len(self.to_add) + len(self.to_rm))

class Installer(TweakPage):
    def __init__(self):
        TweakPage.__init__(self, 
                _('Add/Remove Applications'),
                _('A simple but more effecient method for finding and installing popular packages than the default Add/Remove.'))

        self.package_worker = package_worker

        vbox = gtk.VBox(False, 8)
        self.pack_start(vbox)

        hbox = gtk.HBox(False, 0)
        vbox.pack_start(hbox, False, False, 0)

        combobox = self.create_category()
        self.update_cate_model()
        combobox.set_active(0)
        combobox.connect('changed', self.on_category_changed)
        hbox.pack_end(combobox, False, False, 0)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        vbox.pack_start(sw)

        # create tree view
        self.treeview = AppView()
        self.treeview.update_model(APPS.keys(), APPS)
        self.treeview.sort_model()
        self.treeview.connect('changed', self.on_app_status_changed)
        sw.add(self.treeview)

        # button
        hbox = gtk.HBox(False, 0)
        vbox.pack_end(hbox, False ,False, 0)

        self.button = gtk.Button(stock = gtk.STOCK_APPLY)
        self.button.connect('clicked', self.on_apply_clicked)
        self.button.set_sensitive(False)
        hbox.pack_end(self.button, False, False, 0)

        self.show_all()

    def create_category(self):
        self.cate_model = gtk.ListStore(gobject.TYPE_INT,
                                gobject.TYPE_STRING,
                                gtk.gdk.Pixbuf)

        combobox = gtk.ComboBox(self.cate_model)
        textcell = gtk.CellRendererText()
        pixbufcell = gtk.CellRendererPixbuf()
        combobox.pack_start(pixbufcell, False)
        combobox.pack_start(textcell, True)
        combobox.add_attribute(textcell, 'text', CATE_NAME)
        combobox.add_attribute(pixbufcell, 'pixbuf', CATE_ICON)

        return combobox

    def update_cate_model(self):
        self.cate_model.clear()

        iter = self.cate_model.append()
        self.cate_model.set(iter, 
                CATE_ID, 0,
                CATE_NAME, _('All Categories'),
                CATE_ICON, gtk.gdk.pixbuf_new_from_file(os.path.join(DATA_DIR, 'appcates', 'all.png')))

        for item in CATES_DATA:
            iter = self.cate_model.append()
            self.cate_model.set(iter, 
                    CATE_ID, item[0],
                    CATE_NAME, item[1],
                    CATE_ICON, gtk.gdk.pixbuf_new_from_file(os.path.join(DATA_DIR, 'appcates', item[2])))

    def on_category_changed(self, widget, data = None):
        index = widget.get_active()
        if index:
            liststore = widget.get_model()
            iter = liststore.get_iter(index)
            self.treeview.filter = liststore.get_value(iter, 1)
        else:
            self.treeview.filter = None

        self.treeview.clear_model()
        self.treeview.update_model(APPS.keys(), APPS)

    def deep_update(self):
        package_worker.update_apt_cache(True)
        self.treeview.clear_model()
        self.treeview.update_model(APPS.keys(), APPS)

    def normal_update(self):
        self.treeview.clear_model()
        self.treeview.update_model(APPS.keys(), APPS)

    def on_apply_clicked(self, widget, data = None):
        to_rm = self.treeview.to_rm
        to_add = self.treeview.to_add
        self.package_worker.perform_action(widget.get_toplevel(), to_add, to_rm)

        package_worker.update_apt_cache(True)

        done = package_worker.get_install_status(to_add, to_rm)

        if done:
            self.button.set_sensitive(False)
            InfoDialog(_('Update Successful!')).launch()
        else:
            ErrorDialog(_('Update Failed!')).launch()

        self.treeview.to_add = []
        self.treeview.to_rm = []
        self.treeview.clear_model()
        self.treeview.update_model(APPS.keys(), APPS)

    def on_app_status_changed(self, widget, i):
        if i:
            self.button.set_sensitive(True)
        else:
            self.button.set_sensitive(False)

if __name__ == '__main__':
    from utility import Test
    Test(Installer)
